var path = require('path');
var xlsx = require('no1_xlsx');
var schedule = require("node-schedule");
var pify = require('pify');
var urlExists = pify(require('no1-url-exists'));
var fs = require('fs');
var R = require('ramda');
var clone = require('clone');

if(!global.memory_cache)
{
    global.memory_cache = require('memory-cache');
    global.memory_cache.hao123 = new global.memory_cache.Cache();
}

var watchedFileList = [];
var watchedTaskList = [];

function watchXlsx(path,task)
{
    if(R.indexOf(path)(watchedFileList) == -1)
    {
        watchedFileList.push(path);
        watchedTaskList.push(task);
    }
    fs.watchFile(path, (curr, prev) => {
        let index = R.indexOf(path)(watchedFileList);
        if(index != -1){
            let task = watchedTaskList[index];
            readToMemAndWatch(path, task);
        }else{
            throw new Error("watchedFileList doesn`t match watchedOptionList at path " + path);
        }
    });

}
function checkZiyaCLouds()
{
    let ziya_clouds = global.memory_cache.hao123.get("ziya_clouds");
    for(let c = 0 ; c < ziya_clouds.length;c++){
        let url = ziya_clouds[c]["网址"];
        urlExists(url).then(exists => {
            // console.log("Line " + c + " , " + url +" : " + JSON.stringify(exists)); // true
            ziya_clouds[c]["status"] = exists;
        }).catch(err =>{
            // console.error(err);
        })
    }
}

function scheduleCheckZiyaCLouds()
{
    var rule1     = new schedule.RecurrenceRule();
    // var times1    = [1,11,21,31,41,51];// every 10 seconds
    var times1    = [1];// every 1 minute
    rule1.second  = times1;
    schedule.scheduleJob(rule1, function(){
        checkZiyaCLouds();
    });
}

/**
 * 读取目标 xlsx 文件，缓存至内存cache global.memory_cache.hao123
 * @param path xlsx file fulpath
 * @param tasks Task Array
 *
 * sample 1 :
     * [
     {  // 1st sheet
         sheetNum: 2,
         defVal: "_", // default for empty element
         splitThTd: true,   // if split table head and content
         header:1,  // read row 1 as header
         memKey:{
             th: "device_company_owned_ths",    // head memory key
             td: "device_company_owned_tds"     // body memory key
         }
     },
     {  // 2nd sheet
         sheetNum: 1,
         defVal: "_",
         splitThTd: true,
         header:1,
         memKey:{
             th: "device_customer_provided_ths",
             td: "device_customer_provided_tds"
         }
     }]
     }

     * sample 2 :
     [
     {
         sheetNum:0,    // read sheet 1
         defVal: "",    // default value for empty element
         duplicatedColArr:[0,1,2,3],    // duplicated cols numbers use pre element value in same col
         memKey:"ziya_aid_project"  // memory key
     }]
 */
function readToMemAndWatch(path, tasks)
{
    if(!tasks || !tasks.length || tasks.length <= 0)
        throw new Error("tasks can`t be empty!");
    for(var taskNum = 0; taskNum < tasks.length ;taskNum++){
        let sheetNum = tasks[taskNum].sheetNum;
        let defVal = tasks[taskNum].defVal;
        let duplicatedColArr = tasks[taskNum].duplicatedColArr;
        let header = tasks[taskNum].header;
        let splitThTd = tasks[taskNum].splitThTd;
        let memKey = tasks[taskNum].memKey;
        let xlsx_file = path;
        let workbook = xlsx.readFile(xlsx_file);
        let worksheet = workbook.Sheets[workbook.SheetNames[sheetNum]];
        let option = {};
        if(typeof defVal == "string")
        {
            option.defval = defVal;
        }
        if(typeof header == "number")
        {
            option.header = header;
        }
        if(!!duplicatedColArr && duplicatedColArr.length && duplicatedColArr.length > 0)
        {
            option.duplicate = duplicatedColArr;
        }
        if(!!option.duplicate && option.defval) throw new Error("defValue and duplicatedColArr can`t be both empty!");
        watchXlsx(xlsx_file,tasks);
        let json = xlsx.utils.sheet_to_json(worksheet, clone(option));
        if(typeof splitThTd == "boolean" && splitThTd)
        {
            let ths = json[0];
            let tds = json.slice(1);
            global.memory_cache.hao123.put(memKey.th, ths);
            global.memory_cache.hao123.put(memKey.td, tds);
        } else {
            global.memory_cache.hao123.put(memKey, json);
        }
    }
}

exports.scheduleCheckZiyaCLouds=scheduleCheckZiyaCLouds;
exports.readToMemAndWatch=readToMemAndWatch;



